/*
 * main.vala is a part of snul
 *
 * Snul: is a simple library that implements
 *       a tcp socket client in vala
 *
 * Copyright (C) 2008  Andrea Del Signore
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *
 * Author:
 *     Andrea Del Signore <sejerpz@tin.it>
 */

#include <src/main.h>
#include <stdlib.h>
#include <string.h>
#include "src/snul.h"

struct _SocketMainPrivate {
	SnulTcpClientSocket* _client;
	GMainLoop* _loop;
};
#define SOCKET_MAIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOCKET_TYPE_MAIN, SocketMainPrivate))
enum  {
	SOCKET_MAIN_DUMMY_PROPERTY
};
static void socket_main_client_connected (SnulTcpClientSocket* sender, gint socket_fd, SocketMain* self);
static void socket_main_client_disconnected (SnulTcpClientSocket* sender, SocketMain* self);
static void socket_main_client_error (SnulTcpClientSocket* sender, GError* error, SocketMain* self);
static void socket_main_client_data_received (SnulTcpClientSocket* sender, const char* buffer, gsize len, SocketMain* self);
static gint socket_main_main (int args_length1, char** args);
static gpointer socket_main_parent_class = NULL;
static void socket_main_dispose (GObject * obj);
static int _vala_strcmp0 (const char * str1, const char * str2);


SocketMain* socket_main_new (void) {
	SocketMain * self;
	self = g_object_newv (SOCKET_TYPE_MAIN, 0, NULL);
	return self;
}


static void socket_main_client_connected (SnulTcpClientSocket* sender, gint socket_fd, SocketMain* self) {
	g_return_if_fail (SOCKET_IS_MAIN (self));
	g_return_if_fail (sender == NULL || SNUL_IS_TCP_CLIENT_SOCKET (sender));
	g_debug ("main.vala:38: connected to %s:%s", snul_tcp_client_socket_get_address (sender), snul_tcp_client_socket_get_port (sender));
}


static void socket_main_client_disconnected (SnulTcpClientSocket* sender, SocketMain* self) {
	g_return_if_fail (SOCKET_IS_MAIN (self));
	g_return_if_fail (sender == NULL || SNUL_IS_TCP_CLIENT_SOCKET (sender));
	g_debug ("main.vala:43: disconnected from %s:%s", snul_tcp_client_socket_get_address (sender), snul_tcp_client_socket_get_port (sender));
	g_main_loop_quit (self->priv->_loop);
}


static void socket_main_client_error (SnulTcpClientSocket* sender, GError* error, SocketMain* self) {
	g_return_if_fail (SOCKET_IS_MAIN (self));
	g_return_if_fail (sender == NULL || SNUL_IS_TCP_CLIENT_SOCKET (sender));
	g_debug ("main.vala:49: error received from %s:%s : %s", snul_tcp_client_socket_get_address (sender), snul_tcp_client_socket_get_port (sender), error->message);
}


static void socket_main_client_data_received (SnulTcpClientSocket* sender, const char* buffer, gsize len, SocketMain* self) {
	g_return_if_fail (SOCKET_IS_MAIN (self));
	g_return_if_fail (sender == NULL || SNUL_IS_TCP_CLIENT_SOCKET (sender));
	g_debug ("main.vala:54: data received from %s:%s : %s", snul_tcp_client_socket_get_address (sender), snul_tcp_client_socket_get_port (sender), buffer);
	if (_vala_strcmp0 (buffer, "quit\n") != 0) {
		snul_tcp_client_socket_send (sender, buffer);
	} else {
		snul_tcp_client_socket_disconnect (sender);
	}
}


void socket_main_run (SocketMain* self) {
	GError * inner_error;
	GMainLoop* _tmp0;
	SnulTcpClientSocket* _tmp1;
	g_return_if_fail (SOCKET_IS_MAIN (self));
	inner_error = NULL;
	_tmp0 = NULL;
	self->priv->_loop = (_tmp0 = g_main_loop_new (NULL, FALSE), (self->priv->_loop == NULL ? NULL : (self->priv->_loop = (g_main_loop_unref (self->priv->_loop), NULL))), _tmp0);
	_tmp1 = NULL;
	self->priv->_client = (_tmp1 = snul_tcp_client_socket_new (), (self->priv->_client == NULL ? NULL : (self->priv->_client = (g_object_unref (self->priv->_client), NULL))), _tmp1);
	g_signal_connect_object (self->priv->_client, "connected", ((GCallback) socket_main_client_connected), self, 0);
	g_signal_connect_object (self->priv->_client, "data-received", ((GCallback) socket_main_client_data_received), self, 0);
	g_signal_connect_object (self->priv->_client, "disconnected", ((GCallback) socket_main_client_disconnected), self, 0);
	g_signal_connect_object (self->priv->_client, "error", ((GCallback) socket_main_client_error), self, 0);
	{
		snul_tcp_client_socket_connect (self->priv->_client, "localhost", "2000", &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == SNUL_TCP_SOCKET_ERROR) {
				goto __catch0_snul_tcp_socket_error;
			}
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		g_main_loop_run (self->priv->_loop);
	}
	goto __finally0;
	__catch0_snul_tcp_socket_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("main.vala:74: error: %s", err->message);
		}
	}
	__finally0:
	;
}


static gint socket_main_main (int args_length1, char** args) {
	SocketMain* main;
	gint _tmp0;
	main = socket_main_new ();
	socket_main_run (main);
	return (_tmp0 = 0, (main == NULL ? NULL : (main = (g_object_unref (main), NULL))), _tmp0);
}


int main (int argc, char ** argv) {
	g_type_init ();
	return socket_main_main (argc, argv);
}


static void socket_main_class_init (SocketMainClass * klass) {
	socket_main_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SocketMainPrivate));
	G_OBJECT_CLASS (klass)->dispose = socket_main_dispose;
}


static void socket_main_init (SocketMain * self) {
	self->priv = SOCKET_MAIN_GET_PRIVATE (self);
	self->priv->_client = NULL;
	self->priv->_loop = NULL;
}


static void socket_main_dispose (GObject * obj) {
	SocketMain * self;
	self = SOCKET_MAIN (obj);
	(self->priv->_client == NULL ? NULL : (self->priv->_client = (g_object_unref (self->priv->_client), NULL)));
	(self->priv->_loop == NULL ? NULL : (self->priv->_loop = (g_main_loop_unref (self->priv->_loop), NULL)));
	G_OBJECT_CLASS (socket_main_parent_class)->dispose (obj);
}


GType socket_main_get_type (void) {
	static GType socket_main_type_id = 0;
	if (G_UNLIKELY (socket_main_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (SocketMainClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) socket_main_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SocketMain), 0, (GInstanceInitFunc) socket_main_init };
		socket_main_type_id = g_type_register_static (G_TYPE_OBJECT, "SocketMain", &g_define_type_info, 0);
	}
	return socket_main_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return (str1 != str2);
	}
	return strcmp (str1, str2);
}




